import { ItemComponentTypes, world, system, EntityComponentTypes} from "@minecraft/server";

//TITLE CARD GLOBAL VARIABLES
var titleTargetPlayer;
var titleType = "ERROR";
var initialPosition;

//DIMENSION BASED TITLE CARD TRIGGERS
world.afterEvents.playerDimensionChange.subscribe((event) => {
    //Termination conditions for optimization
    if(event.player.hasTag("beenToNether") && event.player.hasTag("beenToEnd")){
        return;
    }
    if(event.toDimension.id !== "minecraft:nether" && event.toDimension.id !== "minecraft:the_end"){
        return;
    }
    //Global variable prep
    setPlayerVariables(event.player);
    //Dimesion conditions
    if(event.toDimension.id === "minecraft:nether"){
        if(!event.player.hasTag("beenToNether")){
            titleType = "nether";
            system.run(waitForMovement);
        }
        return;
    }
    else if(event.toDimension.id === "minecraft:the_end"){
            if(!event.player.hasTag("beenToEnd")){
                titleType = "end";
                system.run(waitForMovement);
            }
        return;
    }
})

//TITLE CARD FUNCTIONS
//Nether
function netherTitle(){
    titleTargetPlayer.runCommandAsync("playsound lodestone_compass.link_compass_to_lodestone @s ~~~ 0.6 0.3");
    titleTargetPlayer.runCommandAsync("title @s title §l{-THE NETHER-}");
}
function netherSubtitle(){
    titleTargetPlayer.runCommandAsync("playsound random.orb @s ~~~ 0.5 0.6");
    titleTargetPlayer.runCommandAsync("title @s subtitle -- The Nether Wastes --");
}
function netherSounds(){
    titleTargetPlayer.runCommandAsync("playsound liquid.lava @s ~~~ 500.0 0.9");
    titleTargetPlayer.runCommandAsync("playsound portal.trigger @s ~~~ 1.0 0.8");
}

//End
function endTitle(){
    titleTargetPlayer.runCommandAsync("playsound random.enderchestopen @s ~~~ 0.8 1.0");
    titleTargetPlayer.runCommandAsync("title @s title §l{-THE END-}");
}
function endSubtitle(){
    titleTargetPlayer.runCommandAsync("playsound random.orb @s ~~~ 0.5 0.6");
    titleTargetPlayer.runCommandAsync("title @s subtitle -- The Dragon's Perch --");
}
function endSounds(){
    titleTargetPlayer.runCommandAsync("execute as @s at @s if entity @e[type=ender_dragon] run playsound mob.enderdragon.growl @s ~~~ 0.2 0.4");
    titleTargetPlayer.runCommandAsync("execute as @s at @s unless entity @e[type=ender_dragon] run playsound break.amethyst_block @s ~~~ 500.0 0.2");
    titleTargetPlayer.runCommandAsync("execute as @s at @s unless entity @e[type=ender_dragon] run playsound ambient.soulsand_valley.mood @s ~~~ 500.0 0.7");
}

//TITLE CARD CONTROLLER
function playTitle(){
    if(titleType === "nether"){
        titleTargetPlayer.runCommandAsync("tag @s add beenToNether");
        system.run(netherTitle);
        system.runTimeout(netherSubtitle, 12);
        system.runTimeout(netherSounds, 16);
    }
    else if(titleType === "end"){
        titleTargetPlayer.runCommandAsync("tag @s add beenToEnd");
        system.run(endTitle);
        system.runTimeout(endSubtitle, 12);
        system.runTimeout(endSounds, 16);
    }
    else{
        console.warn("error");
    }
}

//STALL TITLE PLAY UNTIL PLAYER MOVES FUNCTION
function waitForMovement(){
    let currPosition = truncateCoords(titleTargetPlayer.location);
    if(initialPosition.x !== currPosition.x || initialPosition.y !== currPosition.y || initialPosition.z !== currPosition.z){
        system.runTimeout(playTitle, 20);
        return;
    }
    system.run(waitForMovement);
}

//TRUNCATE PLAYER COORDINATES FUNCTION
function truncateCoords(position){
    position.x = Math.trunc(position.x);
    position.y = Math.trunc(position.y);
    position.z = Math.trunc(position.z);
    return position;
}

//HANDLE GLOBAL VARIABLES (sorry)
function setPlayerVariables(player){
    titleTargetPlayer = player;
    initialPosition = truncateCoords(player.location);
}